/*
    Solar - a solar system simulator
    Copyright (C) 2004, 2005  Sam Morris <sam@robots.org.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "MainWindow.h"

#include <iostream>
#include <sstream>
#include <cmath>

#include "toString.h"

using namespace solar;

MainWindow::MainWindow(std::string filename)
{
        universe = new Universe(filename);
        universe->setTimeMult(1e6);
        scene1 = new Scene(*universe);

        // Properites
        // 

        this->set_reallocate_redraws(true);
        scene1->set_size_request(600,600);
        
		// gtk_ui_manager handles toolbars and menus

		// main actions
		this->actionGroup = Gtk::ActionGroup::create ();

		this->actionGroup->add (Gtk::Action::create ("UniverseMenu", "Universe"));
		this->actionGroup->add (Gtk::Action::create ("UniverseSaveas", Gtk::Stock::SAVE_AS, "Save as..."), sigc::mem_fun (*this, &MainWindow::on_menu_universe_save));
		this->actionGroup->add (Gtk::Action::create ("UniverseAddplanet", "Add planet"), sigc::mem_fun (*this, &MainWindow::on_menu_universe_addPlanet));
		
		this->actionGroup->add (Gtk::Action::create ("ViewMenu", "View"));
		this->actionGroup->add (Gtk::Action::create ("ViewExaggerate", "Don't exaggerate planet sises"), sigc::mem_fun (*this, &MainWindow::on_menu_view_exaggerate));
		
		this->actionGroup->add (Gtk::Action::create ("DebugMenu", "Debug"));
		this->actionGroup->add (Gtk::Action::create ("DebugPrint", "Print status"), sigc::mem_fun (*this, &MainWindow::on_menu_debug_status));

		this->actionGroup->add (Gtk::Action::create ("timestepSlower10", "< 10x"), sigc::mem_fun (*this, &MainWindow::timestep_slower_10));
		this->actionGroup->add (Gtk::Action::create ("timestepSlower2", "< 2x"), sigc::mem_fun (*this, &MainWindow::timestep_slower_2));
		this->actionGroup->add (Gtk::Action::create ("timestepReal", "1x"), sigc::mem_fun (*this, &MainWindow::timestep_real));
		this->actionGroup->add (Gtk::Action::create ("timestepFaster2", "> 2x"), sigc::mem_fun (*this, &MainWindow::timestep_faster_2));
		this->actionGroup->add (Gtk::Action::create ("timestepFaster10", "> 10x"), sigc::mem_fun (*this, &MainWindow::timestep_faster_10));
		this->actionGroup->add (Gtk::Action::create ("timestepReverse", "Reverse"), sigc::mem_fun (*this, &MainWindow::timestep_reverse));
		
		this->actionGroup->add (Gtk::Action::create ("zoomOut", "Zoom out"), sigc::mem_fun (*this, &MainWindow::on_zoomOut_2));
		this->actionGroup->add (Gtk::Action::create ("zoomIn", "Zoom in"), sigc::mem_fun (*this, &MainWindow::on_zoomIn_2));
		this->actionGroup->add (Gtk::Action::create ("prevPlanet", "Previous Planet"), sigc::mem_fun (*scene1, &Scene::followPrevPlanet));
		this->actionGroup->add (Gtk::Action::create ("nextPlanet", "Next Planet"), sigc::mem_fun (*scene1, &Scene::followNextPlanet));
		
		// edit/run actions
		Gtk::RadioAction::Group group_editrun;
		this->editrunEditAction = Gtk::RadioAction::create (group_editrun, "editrunEdit", "Edit");
		this->actionGroup->add (this->editrunEditAction, sigc::mem_fun (*this, &MainWindow::on_editRun_edit_choose));
		this->editrunRunAction = Gtk::RadioAction::create (group_editrun, "editrunRun", "Run");
		this->actionGroup->add (this->editrunRunAction, sigc::mem_fun (*this, &MainWindow::on_editRun_run_choose));
		
		this->uiManager = Gtk::UIManager::create ();
		this->uiManager->insert_action_group (this->actionGroup);

		try {
			Glib::ustring ui_info = 
				"<ui>"
				"<menubar name='MenuBar'>"
				"	<menu action='UniverseMenu'>"
				"		<menuitem action='UniverseSaveas'/>"
				"		<separator/>"
				"		<menuitem action='UniverseAddplanet'/>"
				"	</menu>"
				"	<menu action='ViewMenu'>"
				"		<menuitem action='ViewExaggerate'/>"
				"	</menu>"
				"	<menu action='DebugMenu'>"
				"		<menuitem action='DebugPrint'/>"
				"	</menu>"
				"</menubar>"
				"<toolbar name='TopBar'>"
				"	<toolitem action='editrunEdit'/>"
				"	<toolitem action='editrunRun'/>"
				"	<separator/>"
				"   <toolitem action='timestepSlower10'/>"
				"   <toolitem action='timestepSlower2'/>"
				"   <toolitem action='timestepReal'/>"
				"   <toolitem action='timestepFaster2'/>"
				"   <toolitem action='timestepFaster10'/>"
				"   <toolitem action='timestepReverse'/>"
				"</toolbar>"
				"<toolbar name='BottomBar'>"
				"   <toolitem action='zoomOut'/>"
				"   <toolitem action='zoomIn'/>"
				"	<separator/>"
				"   <toolitem action='prevPlanet'/>"
				"   <toolitem action='nextPlanet'/>"
				"</toolbar>"
				"</ui>";
			this->uiManager->add_ui_from_string (ui_info);
		} catch (const Glib::Error& ex) {
			std::cerr << "building menus failed: " << ex.what ();
		}

		dynamic_cast <Gtk::Toolbar *> (this->uiManager->get_widget ("/TopBar"))->set_toolbar_style( Gtk::TOOLBAR_TEXT ); // picture, text beside
		dynamic_cast <Gtk::Toolbar *> (this->uiManager->get_widget ("/BottomBar"))->set_toolbar_style( Gtk::TOOLBAR_TEXT ); // picture, text beside

        this->add(vbox);
		vbox.pack_start (*this->uiManager->get_widget ("/MenuBar"), Gtk::PACK_SHRINK);
		vbox.pack_start (*this->uiManager->get_widget ("/TopBar"), Gtk::PACK_SHRINK);
        vbox.add(hpane);
        vbox.pack_end(statusbar, Gtk::PACK_SHRINK);
		vbox.pack_start (*this->uiManager->get_widget ("/BottomBar"), Gtk::PACK_SHRINK);

        hpane.add( (*scene1) );
        //hpane.add2(scene2);

        // Set up the status bar callback
        Glib::signal_timeout().connect(sigc::mem_fun(*this, &MainWindow::update_status), 1000);

        this->show_all();
}

MainWindow::~MainWindow()
{
        delete universe;        universe = NULL;
        delete scene1;          scene1 = NULL;
}

bool MainWindow::update_status()
{
        //statusbar.push( toString( this->universe->getElapsedTime() ) + " seconds elapsed (time rate: " + toString(universe->getTimeMult()) + "x)" );
        statusbar.push( elapsed_time( this->universe->getElapsedTime() ) + " (time rate: " + toString(universe->getTimeMult()) + "x)" );
        return true;
}

/*
void MainWindow::on_menu_universe_new()
{
        this->universe = Universe("");
}

void MainWindow::on_menu_universe_open()
{
        Gtk::FileSelection dialog("Open");

        if (dialog.run() == Gtk::RESPONSE_OK)
                this->universe = Universe( dialog.get_filename() );
}
*/

void MainWindow::on_menu_universe_save()
{
        Gtk::FileSelection dialog("Save As");
  
        if ( dialog.run() == Gtk::RESPONSE_OK)
                this->universe->saveToFile( dialog.get_filename() ); //TODO: error handling

        // Gtk::MessageDialog dialog2(*this, dialog.get_filename()); dialog2.run();
}


// These two functions are a bit awkward. They are both called whenever the
// radio toolbar changes state. However the one chosen by the user is called
// last, so we can hopefully just avoid this little detail.
void MainWindow::on_editRun_edit_choose()
{
        this->universe->setRunning(false);
        this->update_status();
}
void MainWindow::on_editRun_run_choose()
{
        this->universe->setRunning(true);
}


void MainWindow::on_menu_debug_status()
{
        this->universe->printStatus();
}

/*void MainWindow::on_menu_view_ortho()
{
}

void MainWindow::on_menu_view_perspective()
{
}*/

void MainWindow::on_menu_view_exaggerate()
{
        switch (scene1->getExaggerateMode())
        {
        case EXAGGERATE_NONE:
                scene1->setExaggerateMode(EXAGGERATE_LOG);
                break;

        default:
                std::cerr << "unknown scene exaggeration mode; reset to NONE." << std::endl;
                /* fall through */

        case EXAGGERATE_LOG:
                scene1->setExaggerateMode(EXAGGERATE_NONE);
                break;
        }
}

void MainWindow::timestep_slower_2()
{
        universe->setTimeMult( universe->getTimeMult() / 2 );
}

void MainWindow::timestep_slower_10()
{
        universe->setTimeMult( universe->getTimeMult() / 10 );
}

void MainWindow::timestep_real()
{
        universe->setTimeMult( 1 );
}

void MainWindow::timestep_faster_2()
{
        universe->setTimeMult( universe->getTimeMult() * 2 );
}

void MainWindow::timestep_faster_10()
{
        universe->setTimeMult( universe->getTimeMult() * 10 );
}

void MainWindow::timestep_reverse()
{
        universe->setTimeMult( 0 - universe->getTimeMult() );
}

const std::string elapsed_time(long double time)
{
        const int ONE_MINUTE = 60;
        const int ONE_HOUR = 60 * ONE_MINUTE;
        const int ONE_DAY = 24 * ONE_HOUR;
        const int ONE_YEAR = 31556926; // ~365.25 days

        std::ostringstream os;
        long double n;

        n = floor(time/ ONE_YEAR);
        time -= n * ONE_YEAR;
        if (n > 0)
                os << n << " years, ";

        n = floor(time / ONE_DAY);
        time -= n * ONE_DAY;
        if (n > 0)
                os << n << " days, ";

        n = floor(time / ONE_HOUR);
        time -= n * ONE_HOUR;
        if (n > 0)
                os << n << " hours, ";

        n = floor(time / ONE_MINUTE);
        time -= n * ONE_MINUTE;
        if (n > 0)
                os << n << " minutes, ";

        os << time << " seconds.";

        return os.str();
}

void MainWindow::on_zoomOut_2()
{
        scene1->setZoomDistance( scene1->getZoomDistance() * 2 );
}

void MainWindow::on_zoomIn_2()
{
        scene1->setZoomDistance( scene1->getZoomDistance() / 2 );
}

void MainWindow::on_menu_universe_addPlanet()
{
        Planet p("new planet");

        p.setRadius(2e11); // ~earth sized
        p.setPosition( scene1->getCameraPosition() );

        universe->add( p );
}
