/*
    Solar - a solar system simulator
    Copyright (C) 2004, 2005  Sam Morris <sam@robots.org.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef SOLAR_MAINWINDOW_H
#define SOLAR_MAINWINDOW_H

#include "Scene.h"
#include "Universe.h"

#include <gtkmm.h>

namespace solar {

class MainWindow: public Gtk::Window
{
public:
        MainWindow(std::string filename);
        ~MainWindow();

protected:
        // signal handlers
        //
        //virtual void on_menu_universe_new();
        //virtual void on_menu_universe_open();
        virtual void on_menu_universe_save();
        virtual void on_menu_universe_addPlanet();

        virtual void on_menu_debug_status();

        //virtual void on_menu_view_ortho();
        //virtual void on_menu_view_perspective();
        virtual void on_menu_view_exaggerate();

        virtual void on_zoomOut_2();
        virtual void on_zoomIn_2();

        virtual void timestep_slower_10();
        virtual void timestep_slower_2();
        virtual void timestep_real();
        virtual void timestep_faster_2();
        virtual void timestep_faster_10();
        virtual void timestep_reverse();

        virtual void on_editRun_edit_choose();
        virtual void on_editRun_run_choose();

        virtual bool update_status();
   
private:
        // Widgets
        //

        Gtk::VBox vbox;

		Glib::RefPtr<Gtk::UIManager> uiManager;
		Glib::RefPtr<Gtk::ActionGroup> actionGroup;
		Glib::RefPtr<Gtk::RadioAction> editrunEditAction, editrunRunAction;

        Gtk::HPaned hpane;
        Gtk::Statusbar statusbar;

        Universe *universe;
        Scene *scene1; //, scene2, scene3;
};

}

/**
 * Converts a number of seconds into a string such as "23 hours, 12 minutes."
 */
const std::string elapsed_time(long double time);

#endif
