/*
    Solar - a solar system simulator
    Copyright (C) 2004, 2005  Sam Morris <sam@robots.org.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef SOLAR_PLANET_H
#define SOLAR_PLANET_H

/**
 * Models a planet, or other heavenly body.
 */

#include "Vector.h"

#include <string>
#include <libxml++/libxml++.h>

namespace solar {

class Planet
{
public:
        Planet(std::string name);
        Planet(xmlpp::Node *node);      //todo: add const
//        ~Planet();

        void setPosition(const Vector& position);
        const Vector& getPosition() const;
        
        void setVelocity(const Vector& velocity);
        const Vector& getVelocity() const;

        void setColour(const Vector& colour);
        const Vector& getColour() const;

        void setMass(float mass);
        float getMass() const;

        void setRadius(float radius);
        float getRadius() const;

        const std::string& getName() const;
        void setName(const std::string& name);

        Vector getGravity(const Planet& planet) const;

private:
        float radius;
        float mass;
        Vector position;
        Vector velocity;
        Vector colour;
        std::string name;
};

}

using namespace solar;

std::ostream& operator <<(std::ostream& lhs, const Planet& rhs);

#include "Universe.h" //HACK: universe refers to Planet so must be after planet class def

#endif
