/*
    Solar - a solar system simulator
    Copyright (C) 2004, 2005  Sam Morris <sam@robots.org.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "PlanetWindow.h"

#include <sstream>
#include <cmath>

#include "toString.h"

PlanetWindow::PlanetWindow(Planet *planet)
        : Gtk::Dialog("Planet properties", true, true),
          positionTable(3, 2, false),
          velocityTable(3, 2, false)
{
        this->planet = planet;
        
        vBox = this->get_vbox();

        
        vBox->add(nameFrame);
        nameFrame.set_label("Name");
        nameFrame.add(nameEntry);
        nameFrame.show();

        nameEntry.set_text( planet->getName() );
        nameEntry.show();


        vBox->add(doubleHBox);
        doubleHBox.add(massFrame);
        doubleHBox.add(radiusFrame);
        doubleHBox.show();


        massFrame.set_label("Mass");
        massFrame.add(massEntry);
        massFrame.show();

        massEntry.set_text(toString( planet->getMass() ));
        massEntry.show();

        
        radiusFrame.set_label("Radius");
        radiusFrame.add(radiusEntry);
        radiusFrame.show();
        
        radiusEntry.set_text(toString( planet->getRadius() ));
        radiusEntry.show();

        
        vBox->add(vectorHBox);
        vectorHBox.add(positionFrame);
        vectorHBox.add(velocityFrame);
        vectorHBox.show();

       
        positionFrame.set_label("Position");
        positionFrame.add(positionTable);
        positionFrame.show();

        positionTable.show();
        
        positionTable.attach( positionXLabel, 0, 1, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 8, 8);
        positionTable.attach( positionYLabel, 0, 1, 1, 2, Gtk::SHRINK, Gtk::SHRINK, 8, 8);
        positionTable.attach( positionZLabel, 0, 1, 2, 3, Gtk::SHRINK, Gtk::SHRINK, 8, 8);
        
        positionTable.attach( positionXEntry, 1, 2, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 8, 8);
        positionTable.attach( positionYEntry, 1, 2, 1, 2, Gtk::SHRINK, Gtk::SHRINK, 8, 8);
        positionTable.attach( positionZEntry, 1, 2, 2, 3, Gtk::SHRINK, Gtk::SHRINK, 8, 8);

        positionXLabel.set_label("x"); positionXLabel.show();
        positionYLabel.set_label("y"); positionYLabel.show();
        positionZLabel.set_label("z"); positionZLabel.show();

        positionXEntry.set_text(toString( planet->getPosition().getX() ));
        positionXEntry.show();
        positionYEntry.set_text(toString( planet->getPosition().getY() ));
        positionYEntry.show();
        positionZEntry.set_text(toString( planet->getPosition().getZ() ));
        positionZEntry.show();


        velocityFrame.set_label("Velocity");
        velocityFrame.add(velocityTable);
        velocityFrame.show();

        velocityTable.show();
        
        velocityTable.attach( velocityXLabel, 0, 1, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 8, 8);
        velocityTable.attach( velocityYLabel, 0, 1, 1, 2, Gtk::SHRINK, Gtk::SHRINK, 8, 8);
        velocityTable.attach( velocityZLabel, 0, 1, 2, 3, Gtk::SHRINK, Gtk::SHRINK, 8, 8);
        
        velocityTable.attach( velocityXEntry, 1, 2, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 8, 8);
        velocityTable.attach( velocityYEntry, 1, 2, 1, 2, Gtk::SHRINK, Gtk::SHRINK, 8, 8);
        velocityTable.attach( velocityZEntry, 1, 2, 2, 3, Gtk::SHRINK, Gtk::SHRINK, 8, 8);

        velocityXLabel.set_label("x"); velocityXLabel.show();
        velocityYLabel.set_label("y"); velocityYLabel.show();
        velocityZLabel.set_label("z"); velocityZLabel.show();

        velocityXEntry.set_text(toString( planet->getVelocity().getX() ));
        velocityXEntry.show();
        velocityYEntry.set_text(toString( planet->getVelocity().getY() ));
        velocityYEntry.show();
        velocityZEntry.set_text(toString( planet->getVelocity().getZ() ));
        velocityZEntry.show();


        
        vBox->add(colourFrame);
        colourFrame.set_label("Colour");
        colourFrame.add(colourSelection);
        colourFrame.show();

        Gdk::Color c;
        c.set_red(   gushort(planet->getColour().getX() * 65535) );
        c.set_green( gushort(planet->getColour().getY() * 65535) );
        c.set_blue(  gushort(planet->getColour().getZ() * 65535) );
        
        colourSelection.set_previous_color(c);
        colourSelection.set_current_color(c);

        colourSelection.set_has_palette(true);
        colourSelection.show();



        this->add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
        this->add_button(Gtk::Stock::OK,     Gtk::RESPONSE_OK);
}

std::string PlanetWindow::getName()
{
        return nameEntry.get_text();
}

double PlanetWindow::getMass()
{
        return toDouble(massEntry.get_text());
}

double PlanetWindow::getRadius()
{
        return toDouble(radiusEntry.get_text());
}

Vector PlanetWindow::getPosition()
{
        return Vector(toDouble(positionXEntry.get_text()), toDouble(positionYEntry.get_text()), toDouble(positionZEntry.get_text()));
}

Vector PlanetWindow::getVelocity()
{
        return Vector(toDouble(velocityXEntry.get_text()), toDouble(velocityYEntry.get_text()), toDouble(velocityZEntry.get_text()));
}

Vector PlanetWindow::getColour()
{
        return Vector( colourSelection.get_current_color().get_red()   / 65535.0,
                       colourSelection.get_current_color().get_green() / 65535.0,
                       colourSelection.get_current_color().get_blue()  / 65535.0
                         );
}

void PlanetWindow::updatePlanet(Planet& p)
{
        p.setName( getName() );

        p.setMass( getMass() );
        p.setRadius( getRadius() );

        p.setPosition( getPosition() );
        p.setVelocity( getVelocity() );
        p.setColour( getColour() );
}

double solar::toDouble(const std::string& string)
{
        double result;
        std::istringstream is;

        is.str(string);
        is >> result;

        if (is.fail() || result == INFINITY || result == -INFINITY)
                return 0;
        else
                return result;
}
