/*
    Solar - a solar system simulator
    Copyright (C) 2004, 2005  Sam Morris <sam@robots.org.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef SOLAR_SCENE_H
#define SOLAR_SCENE_H

#include <iostream>

#include <gtkmm.h>
#include <gtkglmm.h>

#include <GL/gl.h>
#include <GL/glu.h>

#include "Universe.h"

namespace solar {

typedef enum {
        EXAGGERATE_NONE,
        EXAGGERATE_LOG
} exaggerateMode;

class Scene: public Gtk::GL::DrawingArea
{
public:
        Scene(Universe& universe);
        virtual ~Scene();

        exaggerateMode getExaggerateMode() const;
        void setExaggerateMode(exaggerateMode exaggeration);

        double getZoomDistance() const;
        void setZoomDistance(double zoomDistance);

        Vector getCameraPosition();

        void followNextPlanet();
        void followPrevPlanet();

private:
        double exaggerateRadius(double radius) const;
        
        // view options
        exaggerateMode exaggeration;
        double zoomDistance;

        std::vector<Planet>::iterator followIterator;
        bool followIterator_valid;

        // Signal handlers
        virtual void on_realize();                                 // create
        virtual bool on_configure_event(GdkEventConfigure* event); // resize
        virtual bool on_expose_event(GdkEventExpose* event);       // redraw
        
        virtual bool on_visibility_notify_event(GdkEventVisibility* event);
                // window hidden/occluded

        virtual bool on_button_release_event(GdkEventButton* event);
        virtual bool on_button_press_event(GdkEventButton* event);

        // Signals and their handlers
        SigC::Connection idleConnection;
        virtual bool on_idle();

        // timing/fps evaluation
        Glib::Timer timer;
        int frameCount;

        Universe *universe;
        
        GLUquadricObj *planetQuad;
};

}

#endif
