/*
    Solar - a solar system simulator
    Copyright (C) 2004, 2005  Sam Morris <sam@robots.org.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef SOLAR_VECTOR_H
#define SOLAR_VECTOR_H

#include <iostream>
#include <libxml++/libxml++.h>

/**
 * Models anything that contains three doubles.
 * eg, a vector, a colour, etc
 */

typedef double  vector_t;

namespace solar {

class Vector
{
public:
        Vector(vector_t x, vector_t y, vector_t z);
        Vector(xmlpp::Element *e);

        void addAttrsToXmlElement(xmlpp::Element *colourNode) const;

        vector_t dotProduct(const Vector& vector) const;
        Vector crossProduct(const Vector& vector) const;
        Vector toUnitVector() const;

        vector_t getX() const;
        vector_t getY() const;
        vector_t getZ() const;

        void setX(vector_t x);
        void setY(vector_t y);
        void setZ(vector_t z);

        double getTheta() const;
        double getPhi() const;
        double getPsi() const;

        vector_t getMagnitude() const;
//        vector_t getSeparation(Vector& vector) const;
        
private:
        vector_t x, y, z;
};


}

// Operators
// 
//{
	using namespace solar;

	Vector operator -(const Vector& vector);
	Vector operator +(const Vector& lhs, const Vector& rhs);
	Vector operator -(const Vector& lhs, const Vector& rhs);
	Vector operator *(const Vector& lhs, vector_t rhs);

	std::ostream& operator <<(std::ostream& lhs, const Vector& rhs);
//}

#endif
