/*
    Solar - a solar system simulator
    Copyright (C) 2004, 2005  Sam Morris <sam@robots.org.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "main.h"

#include <iostream>
#include <getopt.h>
#include <cmath>

#include <gtkmm.h>

#include "Vector.h"
#include "Planet.h"
#include "PlanetWindow.h"

using namespace std;

int main(int argc, char* argv[])
{
        string filename = "";
        
        char c;
        while ( ( c = getopt( argc, argv, "t::f:b:" ) ) != -1 )
        {
                switch (c)
                {
                case 't':
                        if (optarg == '\0')
                                test("");
                        else
                                test( string(optarg) );

                        return 0;
                        break;
                        
                case 'f':
                        filename = string(optarg);
                        break;

                case 'b':
                        if (optarg != '\0')
                                batch( string(optarg) );

                }
        }
        
        Gtk::Main kit(argc, argv);
        Gtk::GL::init(argc, argv);

        //int major, minor;
        //Gdk::GL::query_version(major, minor);
        //cerr << "OpenGL v" << major << "." << minor << endl;

        solar::MainWindow mainWindow(filename);
        kit.run(mainWindow);

        return 0;
}

void test(const std::string& name)
{
        using namespace solar;

        cout << "Solar test suite" << endl;

        if (name == "vector")
        {
                cout << endl << "Vector." << endl;

                Vector a(1, 0, 0);
                Vector b(0, 1, 0);
                //Vector c = Vector(cos(p), sin(p), 0);
                Vector c = Vector(1, 1, 0).toUnitVector();
                Vector d = Vector(0, 0, 1);
                Vector e = Vector(0, 0, -1);

                Vector v(1, 2, 3);
                Vector w(-1, 9.5, 3);

                cout.precision(3);
        
                cout << "  v = " << v << endl;
                cout << " -v = " << -v << endl;
                cout << " 2v = " << (v * 2) << endl;
                cout << "v+v = " << (v + v) << endl;
                cout << a << "•" << a << " = " << a.dotProduct(a) << endl;
                cout << "|v| = " << v.getMagnitude() << endl;

                cout << "  w = " << w << endl;
        
                cout << "v+w = " << v+w << endl;
                cout << "w-v = " << w-v << endl;

                cout << a << "×" <<  b << " = " << a.crossProduct(b) << endl;
                
                cout << a << "•" <<  b << " = " << a.dotProduct(b) << endl;
                cout << a << "•" <<  a << " = " << a.dotProduct(a) << endl;
                cout << a << "•" << -a << " = " << a.dotProduct(-a) << endl;
                cout << b << "•" << -b << " = " << b.dotProduct(-b) << endl;
                cout << d << "•" <<  e << " = " << d.dotProduct(e) << endl;

                cout << "cos-1( " << a << "•" << c << " ) = " << acos( a.dotProduct(c) ) / (2 * M_PI) * 360 << "°" << endl;
        
                cout << "v×w = " << v.crossProduct(w) << endl;
                cout << "w×v = " << w.crossProduct(v) << endl;

                cout << "v•w = " << v.dotProduct(w) << endl;
                cout << "w•v = " << w.dotProduct(v) << endl;
        }
        else if (name == "angles")
        {
                cout << endl << "theta test" << endl;
                double x, y, z;

                for (int i = 0; i <= 360; i += 10)
                {
                        x = 10 * cos( (i/360.0) * (2 * M_PI) );
                        y = 10 * sin( (i/360.0) * (2 * M_PI) );

                        Vector p = Vector(x, y, 0);
                        cout << p << "; theta = " << p.getTheta() / (2*M_PI) * 360 << " degrees" << endl;
                }

                cout << endl << "phi test" << endl;
                for (int i = 0; i <= 360; i += 10)
                {
                        x = 10 * sin( (i/360.0) * (2 * M_PI) );
                        y = 10 * cos( (i/360.0) * (2 * M_PI) );

                        Vector p = Vector(x, y, 0);
                        cout << p << "; phi = " << p.getPhi() / (2*M_PI) * 360 << " degrees" << endl;
                }

                cout << endl << "psi test" << endl;
                for (int i = 0; i <= 360; i += 10)
                {
                        x = 10 * sin( (i/360.0) * (2 * M_PI) );
                        z = 10 * cos( (i/360.0) * (2 * M_PI) );

                        Vector p = Vector(x, 0, z);
                        cout << p << "; psi = " << p.getPsi() / (2*M_PI) * 360 << " degrees" << endl;
                }
        }
        else if (name == "planet")
        {
                cout << endl << "Planet." << endl;
                
                Planet sol("Sol");
                sol.setPosition( Vector(0, 0, 0) );
                sol.setMass(1.99e30);
                sol.setRadius(71492 * 1000);
                cout << sol << endl;

                Planet jupiter("Jupiter");
                jupiter.setPosition( Vector(7.78e11 , 0, 0) );
                jupiter.setMass(1.9e27);
                jupiter.setRadius(7.15e7);
                cout << jupiter << endl;

                Planet earth("Earth");
                earth.setPosition( Vector(1.496e11, 0, 0) );
                earth.setMass(5.98e24);
                earth.setRadius(6.38e6);
                cout << earth << endl;

                cout << "Sol.gravity(Earth) = " << sol.getGravity(earth) << endl;
                cout << '\t' << "|F| = " << sol.getGravity(earth).getMagnitude() << endl;

                cout << "Earth.gravity(Sol) = " << earth.getGravity(sol) << endl;
                cout << '\t' << "|F| = " << earth.getGravity(sol).getMagnitude() << endl;
        
                cout << "Sol <> Jupiter = " << sol.getGravity(jupiter) << endl;
                cout << '\t' << "|F| = " << sol.getGravity(jupiter).getMagnitude() << endl;

                cout << "Earth <> Jupiter = " << earth.getGravity(jupiter) << endl;
                cout << '\t' << "|F| = " << earth.getGravity(jupiter).getMagnitude() << endl;
        }
        else if (name == "planetwindow")
        {
                cout << "PlanetWindow." << endl;

                Planet p("Narn");
                p.setMass(1.23e4);
                p.setRadius(5.67e8);
                p.setPosition(Vector(1, 2, 3));
                p.setVelocity(Vector(4, 5, 6));
                p.setColour(Vector(0.3, 0.5, 0.7));

                Gtk::Main kit(NULL, NULL);
                PlanetWindow w(&p);

                if (w.run() == Gtk::RESPONSE_OK)
                {
                        w.updatePlanet(p);
                        cout << p << endl;
                }
        }
        else
        {
                cout << "Unknown test '" << name << "'. Available tests:" << endl;
                cout << "\tvector" << endl;
                cout << "\tangles" << endl;
                cout << "\tplanet" << endl;
                cout << "\tplanetwindow" << endl;
        }
}

void batch(const std::string& name)
{
        int timestep = 60*60 * 6; // 6 hours
        int iterations = 60*60*24*365*4 / timestep;

        Universe u(name, timestep); // timestep = 6 hours

        std::cerr.precision(0);

        for (int i=0; i < iterations; i++)
        {
                if (i % (iterations/20) == 0) // every 5%, print out the %age
                        std::cerr << (i * 1.0) / iterations * 100 << "%" << std::endl;

                u.think();

                for (std::vector<Planet>::iterator i = u.planets.begin(); i < u.planets.end(); i++)
                        std::cout << i->getName() << " " << i->getPosition().getX() << " " << i->getPosition().getY() << " " << i->getPosition().getZ() << std::endl;
        }

        exit(0);
}
